@@ -3,7 +3,7 @@
  *
  * See the README file for copyright information and how to reach the author.
  *
- * $Id: i18n.c 1.4 2004/01/16 13:28:23 kls Exp $
+ * $Id: i18n.c 1.6 2006/04/22 09:30:57 kls Exp $
  */
 
 #include "i18n.h"
@@ -17,7 +17,7 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     "",// TODO
-    "",// TODO
+    "Hei",
     "",// TODO
     "",// TODO
     "",// TODO
@@ -26,6 +26,10 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     "!",
+    "",// TODO
+    "",// TODO
+    "",// TODO
+    "Ahoj",
   },
   { "Hello world!",
     "Hallo Welt!",
@@ -35,7 +39,7 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     "",// TODO
-    "",// TODO
+    "Hei maailma!",
     "",// TODO
     "",// TODO
     "",// TODO
@@ -44,6 +48,10 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     " !",
+    "",// TODO
+    "",// TODO
+    "",// TODO
+    "Ahoj svte!",
   },
   { "Howdy folks!",
     "Tach zusammen!",
@@ -53,7 +61,7 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     "",// TODO
-    "",// TODO
+    "Mit kuuluu?",
     "",// TODO
     "",// TODO
     "",// TODO
@@ -62,6 +70,10 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     " ?",
+    "",// TODO
+    "",// TODO
+    "",// TODO
+    "Nazdar vespolek!",
   },
   { "A friendly greeting",
     "Ein freundlicher Gru",
@@ -71,7 +83,7 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     "",// TODO
-    "",// TODO
+    "Ystvllisin terveisin",
     "",// TODO
     "",// TODO
     "",// TODO
@@ -80,6 +92,10 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     " ",
+    "",// TODO
+    "",// TODO
+    "",// TODO
+    "Ptelsk pozdrav",
   },
   { "Greeting time (s)",
     "Dauer des Grues (s)",
@@ -89,7 +105,7 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     "",// TODO
-    "",// TODO
+    "Tervehdysaika (s)",
     "",// TODO
     "",// TODO
     "",// TODO
@@ -98,6 +114,10 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     " ",
+    "",// TODO
+    "",// TODO
+    "",// TODO
+    "as pozdravu (s)",
   },
   { "Use alternate greeting",
     "Alternativen Gru verwenden",
@@ -107,7 +127,7 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     "",// TODO
-    "",// TODO
+    "Kyt vaihtoehtoista tervehdyst",
     "",// TODO
     "",// TODO
     "",// TODO
@@ -116,6 +136,10 @@ const tI18nPhrase Phrases[] = {
     "",// TODO
     "",// TODO
     " ",
+    "",// TODO
+    "",// TODO
+    "",// TODO
+    "Pout alternativn pozdrav",
   },
   { NULL }
   };
