@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbdevice.h 1.39 2006/04/01 14:18:59 kls Exp $
+ * $Id: dvbdevice.h 1.41 2006/05/28 15:05:19 kls Exp $
  */
 
 #ifndef __DVBDEVICE_H
@@ -104,14 +104,19 @@ protected:
 
 private:
   bool digitalAudio;
-  static bool setTransferModeForDolbyDigital;
+  static int setTransferModeForDolbyDigital;
 protected:
   virtual int GetAudioChannelDevice(void);
   virtual void SetAudioChannelDevice(int AudioChannel);
   virtual void SetVolumeDevice(int Volume);
   virtual void SetDigitalAudioDevice(bool On);
 public:
-  static void SetTransferModeForDolbyDigital(bool On);
+  static void SetTransferModeForDolbyDigital(int Mode);
+         ///< Controls how the DVB device handles Transfer Mode when replaying
+         ///< Dolby Digital audio.
+         ///< 0 = don't set "audio bypass" in driver/firmware, don't force Transfer Mode
+         ///< 1 = set "audio bypass" in driver/firmware, force Transfer Mode (default)
+         ///< 2 = don't set "audio bypass" in driver/firmware, force Transfer Mode
 
 // Player facilities
 
