@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: dvbplayer.c 1.41 2005/10/31 12:33:48 kls Exp $
+ * $Id: dvbplayer.c 1.45 2006/04/17 12:45:48 kls Exp $
  */
 
 #include "dvbplayer.h"
@@ -15,7 +15,7 @@
 #include "thread.h"
 #include "tools.h"
 
-// --- cBackTrace ----------------------------------------------------------
+// --- cBackTrace ------------------------------------------------------------
 
 #define AVG_FRAME_SIZE 15000         // an assumption about the average frame size
 #define DVB_BUF_SIZE   (256 * 1024)  // an assumption about the dvb firmware buffer size
@@ -80,6 +80,8 @@ private:
   int length;
   bool hasData;
   cCondWait newSet;
+  cCondVar newDataCond;
+  cMutex newDataMutex;
 protected:
   void Action(void);
 public:
@@ -88,6 +90,7 @@ public:
   void Clear(void);
   int Read(cUnbufferedFile *File, uchar *Buffer, int Length);
   bool Reading(void) { return buffer; }
+  bool WaitForDataMs(int msToWait);
   };
 
 cNonBlockingFileReader::cNonBlockingFileReader(void)
@@ -150,8 +153,11 @@ void cNonBlockingFileReader::Action(void)
            int r = f->Read(buffer + length, wanted - length);
            if (r >= 0) {
               length += r;
-              if (!r || length == wanted) // r == 0 means EOF
+              if (!r || length == wanted) { // r == 0 means EOF
+                 cMutexLock NewDataLock(&newDataMutex);
                  hasData = true;
+                 newDataCond.Broadcast();
+                 }
               }
            else if (r < 0 && FATALERRNO) {
               LOG_ERROR;
@@ -164,6 +170,14 @@ void cNonBlockingFileReader::Action(void)
         }
 }
 
+bool cNonBlockingFileReader::WaitForDataMs(int msToWait)
+{
+  cMutexLock NewDataLock(&newDataMutex);
+  if (hasData)
+     return true;
+  return newDataCond.TimedWait(newDataMutex, msToWait);
+}
+
 // --- cDvbPlayer ------------------------------------------------------------
 
 #define PLAYERBUFSIZE  MEGABYTE(1)
@@ -362,10 +376,14 @@ void cDvbPlayer::Action(void)
   nonBlockingFileReader = new cNonBlockingFileReader;
   int Length = 0;
   bool Sleep = false;
+  bool WaitingForData = false;
 
   while (Running() && (NextFile() || readIndex >= 0 || ringBuffer->Available() || !DeviceFlush(100))) {
         if (Sleep) {
-           cCondWait::SleepMs(3); // this keeps the CPU load low
+           if (WaitingForData)
+              nonBlockingFileReader->WaitForDataMs(3); // this keeps the CPU load low, but reacts immediately on new data
+           else
+              cCondWait::SleepMs(3); // this keeps the CPU load low
            Sleep = false;
            }
         cPoller Poller;
@@ -381,12 +399,19 @@ void cDvbPlayer::Action(void)
                     if (playMode == pmFast || (playMode == pmSlow && playDir == pdBackward)) {
                        uchar FileNumber;
                        int FileOffset;
-                       int Index = index->GetNextIFrame(readIndex, playDir == pdForward, &FileNumber, &FileOffset, &Length, true);
+                       bool TimeShiftMode = index->IsStillRecording();
+                       int Index = index->GetNextIFrame(readIndex, playDir == pdForward, &FileNumber, &FileOffset, &Length, TimeShiftMode);
                        if (Index >= 0) {
                           if (!NextFile(FileNumber, FileOffset))
                              continue;
                           }
                        else {
+                          if (!TimeShiftMode && playDir == pdForward) {
+                             // hit end of recording: signal end of file but don't change playMode
+                             readIndex = -1;
+                             eof = true;
+                             continue;
+                             }
                           // hit begin of recording: wait for device buffers to drain
                           // before changing play mode:
                           if (!DeviceFlush(100))
@@ -423,11 +448,14 @@ void cDvbPlayer::Action(void)
                     }
                  int r = nonBlockingFileReader->Read(replayFile, b, Length);
                  if (r > 0) {
+                    WaitingForData = false;
                     readFrame = new cFrame(b, -r, ftUnknown, readIndex); // hands over b to the ringBuffer
                     b = NULL;
                     }
                  else if (r == 0)
                     eof = true;
+                 else if (r < 0 && errno == EAGAIN)
+                    WaitingForData = true;
                  else if (r < 0 && FATALERRNO) {
                     LOG_ERROR;
                     break;
@@ -621,7 +649,7 @@ int cDvbPlayer::SkipFrames(int Frames)
      int Current, Total;
      GetIndex(Current, Total, true);
      int OldCurrent = Current;
-     // As GetNextIFrame() increments/decrements at least once, the 
+     // As GetNextIFrame() increments/decrements at least once, the
      // destination frame (= Current + Frames) must be adjusted by
      // -1/+1 respectively.
      Current = index->GetNextIFrame(Current + Frames + (Frames > 0 ? -1 : 1), Frames > 0);
