@@ -7,7 +7,7 @@
  * Original version (as used in VDR before 1.3.0) written by
  * Robert Schneider <Robert.Schneider@web.de> and Rolf Hakenes <hakenes@hippomi.de>.
  *
- * $Id: epg.c 1.72 2006/04/22 12:02:47 kls Exp $
+ * $Id: epg.c 1.81 2006/10/28 09:12:42 kls Exp $
  */
 
 #include "epg.h"
@@ -87,7 +87,8 @@ void cComponents::SetComponent(int Index, uchar Stream, uchar Type, const char *
 tComponent *cComponents::GetComponent(int Index, uchar Stream, uchar Type)
 {
   for (int i = 0; i < numComponents; i++) {
-      if (components[i].stream == Stream && components[i].type == Type) {
+      // In case of an audio stream the 'type' check actually just distinguishes between "normal" and "Dolby Digital":
+      if (components[i].stream == Stream && (Stream != 2 || (components[i].type < 5) == (Type < 5))) {
          if (!Index--)
             return &components[i];
          }
@@ -227,7 +228,7 @@ bool cEvent::HasTimer(void) const
 
 bool cEvent::IsRunning(bool OrAboutToStart) const
 {
-  return SeenWithin(RUNNINGSTATUSTIMEOUT) && runningStatus >= (OrAboutToStart ? SI::RunningStatusStartsInAFewSeconds : SI::RunningStatusPausing);
+  return runningStatus >= (OrAboutToStart ? SI::RunningStatusStartsInAFewSeconds : SI::RunningStatusPausing);
 }
 
 cString cEvent::GetDateString(void) const
@@ -327,8 +328,10 @@ bool cEvent::Read(FILE *f, cSchedule *Schedule)
                              cEvent *newEvent = NULL;
                              if (Event)
                                 DELETENULL(Event->components);
-                             if (!Event)
+                             if (!Event) {
                                 Event = newEvent = new cEvent(EventID);
+                                Event->seen = 0;
+                                }
                              if (Event) {
                                 Event->SetTableID(TableID);
                                 Event->SetStartTime(StartTime);
@@ -433,18 +436,6 @@ void ReportEpgBugFixStats(bool Reset)
 
 void cEvent::FixEpgBugs(void)
 {
-  // VDR can't usefully handle newline characters in the title and shortText of EPG
-  // data, so let's always convert them to blanks (independent of the setting of EPGBugfixLevel):
-  strreplace(title, '\n', ' ');
-  strreplace(shortText, '\n', ' ');
-  // Same for control characters:
-  strreplace(title, '\x86', ' ');
-  strreplace(title, '\x87', ' ');
-  strreplace(shortText, '\x86', ' ');
-  strreplace(shortText, '\x87', ' ');
-  strreplace(description, '\x86', ' ');
-  strreplace(description, '\x87', ' ');
-
   if (isempty(title)) {
      // we don't want any "(null)" titles
      title = strcpyrealloc(title, tr("No title"));
@@ -452,7 +443,7 @@ void cEvent::FixEpgBugs(void)
      }
 
   if (Setup.EPGBugfixLevel == 0)
-     return;
+     goto Final;
 
   // Some TV stations apparently have their own idea about how to fill in the
   // EPG data. Let's fix their bugs as good as we can:
@@ -526,7 +517,7 @@ void cEvent::FixEpgBugs(void)
      }
 
   if (Setup.EPGBugfixLevel <= 1)
-     return;
+     goto Final;
 
   // Some channels apparently try to do some formatting in the texts,
   // which is a bad idea because they have no way of knowing the width
@@ -572,7 +563,7 @@ void cEvent::FixEpgBugs(void)
   strreplace(description, '`', '\'');
 
   if (Setup.EPGBugfixLevel <= 2)
-     return;
+     goto Final;
 
   // The stream components have a "description" field which some channels
   // apparently have no idea of how to set correctly:
@@ -636,6 +627,20 @@ void cEvent::FixEpgBugs(void)
            }
          }
      }
+
+Final:
+
+  // VDR can't usefully handle newline characters in the title and shortText of EPG
+  // data, so let's always convert them to blanks (independent of the setting of EPGBugfixLevel):
+  strreplace(title, '\n', ' ');
+  strreplace(shortText, '\n', ' ');
+  // Same for control characters:
+  strreplace(title, '\x86', ' ');
+  strreplace(title, '\x87', ' ');
+  strreplace(shortText, '\x86', ' ');
+  strreplace(shortText, '\x87', ' ');
+  strreplace(description, '\x86', ' ');
+  strreplace(description, '\x87', ' ');
 }
 
 // --- cSchedule -------------------------------------------------------------
@@ -643,7 +648,7 @@ void cEvent::FixEpgBugs(void)
 cSchedule::cSchedule(tChannelID ChannelID)
 {
   channelID = ChannelID;
-  hasRunning = false;;
+  hasRunning = false;
   modified = 0;
   presentSeen = 0;
 }
@@ -659,6 +664,8 @@ cEvent *cSchedule::AddEvent(cEvent *Event)
 void cSchedule::DelEvent(cEvent *Event)
 {
   if (Event->schedule == this) {
+     if (hasRunning && Event->IsRunning())
+        ClrRunningStatus();
      UnhashEvent(Event);
      events.Del(Event);
      }
@@ -710,12 +717,12 @@ const cEvent *cSchedule::GetFollowingEvent(void) const
 
 const cEvent *cSchedule::GetEvent(tEventID EventID, time_t StartTime) const
 {
-  // Returns either the event info with the given EventID or, if that one can't
-  // be found, the one with the given StartTime (or NULL if neither can be found)
-  cEvent *pt = eventsHashID.Get(EventID);
-  if (!pt && StartTime > 0) // 'StartTime < 0' is apparently used with NVOD channels
-     pt = eventsHashStartTime.Get(StartTime);
-  return pt;
+  // Returns the event info with the given StartTime or, if no actual StartTime
+  // is given, the one with the given EventID.
+  if (StartTime > 0) // 'StartTime < 0' is apparently used with NVOD channels
+     return eventsHashStartTime.Get(StartTime);
+  else
+     return eventsHashID.Get(EventID);
 }
 
 const cEvent *cSchedule::GetEventAround(time_t Time) const
@@ -734,16 +741,19 @@ const cEvent *cSchedule::GetEventAround(time_t Time) const
 
 void cSchedule::SetRunningStatus(cEvent *Event, int RunningStatus, cChannel *Channel)
 {
+  hasRunning = false;
   for (cEvent *p = events.First(); p; p = events.Next(p)) {
       if (p == Event) {
-         if (p->RunningStatus() > SI::RunningStatusNotRunning || RunningStatus > SI::RunningStatusNotRunning)
+         if (p->RunningStatus() > SI::RunningStatusNotRunning || RunningStatus > SI::RunningStatusNotRunning) {
             p->SetRunningStatus(RunningStatus, Channel);
+            break;
+            }
          }
       else if (RunningStatus >= SI::RunningStatusPausing && p->StartTime() < Event->StartTime())
          p->SetRunningStatus(SI::RunningStatusNotRunning);
+      if (p->RunningStatus() >= SI::RunningStatusPausing)
+         hasRunning = true;
       }
-  if (RunningStatus >= SI::RunningStatusPausing)
-     hasRunning = true;
 }
 
 void cSchedule::ClrRunningStatus(cChannel *Channel)
@@ -768,6 +778,14 @@ void cSchedule::ResetVersions(void)
 void cSchedule::Sort(void)
 {
   events.Sort();
+  // Make sure there are no RunningStatusUndefined before the currently running event:
+  if (hasRunning) {
+     for (cEvent *p = events.First(); p; p = events.Next(p)) {
+         if (p->RunningStatus() >= SI::RunningStatusPausing)
+            break;
+         p->SetRunningStatus(SI::RunningStatusNotRunning);
+         }
+     }
 }
 
 void cSchedule::DropOutdated(time_t SegmentStart, time_t SegmentEnd, uchar TableID, uchar Version)
@@ -783,6 +801,8 @@ void cSchedule::DropOutdated(time_t SegmentStart, time_t SegmentEnd, uchar Table
                   // We can't delete the event right here because a timer might have
                   // a pointer to it, so let's set its id and start time to 0 to have it
                   // "phased out":
+                  if (hasRunning && p->IsRunning())
+                     ClrRunningStatus();
                   UnhashEvent(p);
                   p->eventID = 0;
                   p->startTime = 0;
@@ -1023,6 +1043,9 @@ cSchedule *cSchedules::AddSchedule(tChannelID ChannelID)
   if (!p) {
      p = new cSchedule(ChannelID);
      Add(p);
+     cChannel *channel = Channels.GetByChannelID(ChannelID);
+     if (channel)
+        channel->schedule = p;
      }
   return p;
 }
