@@ -154,9 +154,9 @@ SuckGlyphsFromServer(Display * dpy, Font font)
     character.byte2 = (i + fontinfo->min_char_or_byte2) & 255;
     character.byte1 = (i + fontinfo->min_char_or_byte2) >> 8;
 
-    /* XXX we could use XDrawImageString16 which would also paint the backing 
+    /* XXX we could use XDrawImageString16 which would also paint the backing
 
-       rectangle but X server bugs in some scalable font rasterizers makes it 
+       rectangle but X server bugs in some scalable font rasterizers makes it
 
        more effective to do XFillRectangles to clear the pixmap and
        XDrawImage16 for the text.  */
@@ -371,7 +371,7 @@ main(int argc, char *argv[])
     exit(1);
   }
 
-  printf("%s[][%d] = {\n", varname, fontinfo->max_ascent + fontinfo->max_descent + 2);
+  printf("static const %s[][%d] = {\n", varname, fontinfo->max_ascent + fontinfo->max_descent + 2);
   for (c = 32; c < 256; c++) {
       getMetric(fontinfo, c, &tgi);
       printGlyph(fontinfo, c);
