@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: menuitems.h 1.12 2005/11/11 13:26:51 kls Exp $
+ * $Id: menuitems.h 1.20 2006/04/14 10:01:47 kls Exp $
  */
 
 #ifndef __MENUITEMS_H
@@ -17,7 +17,6 @@ extern const char *FileNameChars;
 class cMenuEditItem : public cOsdItem {
 private:
   char *name;
-  char *value;
 public:
   cMenuEditItem(const char *Name);
   ~cMenuEditItem();
@@ -28,9 +27,10 @@ class cMenuEditIntItem : public cMenuEditItem {
 protected:
   int *value;
   int min, max;
+  const char *minString, *maxString;
   virtual void Set(void);
 public:
-  cMenuEditIntItem(const char *Name, int *Value, int Min = 0, int Max = INT_MAX);
+  cMenuEditIntItem(const char *Name, int *Value, int Min = 0, int Max = INT_MAX, const char *MinString = NULL, const char *MaxString = NULL);
   virtual eOSState ProcessKey(eKeys Key);
   };
 
@@ -44,12 +44,12 @@ public:
 
 class cMenuEditBitItem : public cMenuEditBoolItem {
 protected:
-  int *value;
+  uint *value;
+  uint mask;
   int bit;
-  int mask;
   virtual void Set(void);
 public:
-  cMenuEditBitItem(const char *Name, int *Value, int Mask, const char *FalseString = NULL, const char *TrueString = NULL);
+  cMenuEditBitItem(const char *Name, uint *Value, uint Mask, const char *FalseString = NULL, const char *TrueString = NULL);
   };
 
 class cMenuEditNumItem : public cMenuEditItem {
@@ -77,14 +77,22 @@ public:
 
 class cMenuEditStrItem : public cMenuEditItem {
 private:
+  char *orgValue;
   char *value;
   int length;
   char *allowed;
   int pos;
   bool insert, newchar, uppercase;
+  const char *charMap;
+  const char *currentChar;
+  eKeys lastKey;
+  cTimeMs autoAdvanceTimeout;
   void SetHelpKeys(void);
+  void AdvancePos(void);
   virtual void Set(void);
   char Inc(char c, bool Up);
+protected:
+  bool InEditMode(void) { return pos >= 0; }
 public:
   cMenuEditStrItem(const char *Name, char *Value, int Length, const char *Allowed);
   ~cMenuEditStrItem();
@@ -102,9 +110,10 @@ public:
 
 class cMenuEditChanItem : public cMenuEditIntItem {
 protected:
+  const char *noneString;
   virtual void Set(void);
 public:
-  cMenuEditChanItem(const char *Name, int *Value);
+  cMenuEditChanItem(const char *Name, int *Value, const char *NoneString = NULL);
   virtual eOSState ProcessKey(eKeys Key);
   };
 
@@ -112,7 +121,7 @@ class cMenuEditTranItem : public cMenuEditChanItem {
 private:
   int number;
   int *source;
-  int transponder;
+  int *transponder;
 public:
   cMenuEditTranItem(const char *Name, int *Value, int *Source);
   virtual eOSState ProcessKey(eKeys Key);
