@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.c 1.148 2006/04/29 13:22:20 kls Exp $
+ * $Id: recording.c 1.151 2006/10/07 12:46:22 kls Exp $
  */
 
 #include "recording.h"
@@ -941,13 +941,18 @@ bool cRecordings::StateChanged(int &State)
 
 void cRecordings::TouchUpdate(void)
 {
+  bool needsUpdate = NeedsUpdate();
   TouchFile(UpdateFileName());
-  lastUpdate = time(NULL); // make sure we don't tigger ourselves
+  if (!needsUpdate)
+     lastUpdate = time(NULL); // make sure we don't tigger ourselves
 }
 
 bool cRecordings::NeedsUpdate(void)
 {
-  return lastUpdate < LastModifiedTime(UpdateFileName());
+  time_t lastModified = LastModifiedTime(UpdateFileName());
+  if (lastModified > time(NULL))
+     return false; // somebody's clock isn't running correctly
+  return lastUpdate < lastModified;
 }
 
 bool cRecordings::Update(bool Wait)
@@ -970,7 +975,7 @@ cRecording *cRecordings::GetByName(const char *FileName)
   return NULL;
 }
 
-void cRecordings::AddByName(const char *FileName)
+void cRecordings::AddByName(const char *FileName, bool TriggerUpdate)
 {
   LOCK_THREAD;
   cRecording *recording = GetByName(FileName);
@@ -978,7 +983,8 @@ void cRecordings::AddByName(const char *FileName)
      recording = new cRecording(FileName);
      Add(recording);
      ChangeState();
-     TouchUpdate();
+     if (TriggerUpdate)
+        TouchUpdate();
      }
 }
 
