@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: skinclassic.c 1.12 2005/05/16 10:45:07 kls Exp $
+ * $Id: skinclassic.c 1.15 2006/03/31 13:59:50 kls Exp $
  */
 
 #include "skinclassic.h"
@@ -169,6 +169,8 @@ public:
   virtual void SetEvent(const cEvent *Event);
   virtual void SetRecording(const cRecording *Recording);
   virtual void SetText(const char *Text, bool FixedFont);
+  virtual int GetTextAreaWidth(void) const;
+  virtual const cFont *GetTextAreaFont(bool FixedFont) const;
   virtual void Flush(void);
   };
 
@@ -357,11 +359,20 @@ void cSkinClassicDisplayMenu::SetRecording(const cRecording *Recording)
 
 void cSkinClassicDisplayMenu::SetText(const char *Text, bool FixedFont)
 {
-  const cFont *font = cFont::GetFont(FixedFont ? fontFix : fontOsd);
-  textScroller.Set(osd, x0, y2, x1 - x0 - 2 * ScrollWidth, y3 - y2, Text, font, Theme.Color(clrMenuText), Theme.Color(clrBackground));
+  textScroller.Set(osd, x0, y2, GetTextAreaWidth(), y3 - y2, Text, GetTextAreaFont(FixedFont), Theme.Color(clrMenuText), Theme.Color(clrBackground));
   SetScrollbar();
 }
 
+int cSkinClassicDisplayMenu::GetTextAreaWidth(void) const
+{
+  return x1 - x0 - 2 * ScrollWidth;
+}
+
+const cFont *cSkinClassicDisplayMenu::GetTextAreaFont(bool FixedFont) const
+{
+  return cFont::GetFont(FixedFont ? fontFix : fontOsd);
+}
+
 void cSkinClassicDisplayMenu::Flush(void)
 {
   cString date = DayDateTime();
@@ -378,7 +389,6 @@ private:
   int x0, x1;
   int y0, y1, y2, y3;
   int lastCurrentWidth;
-  bool message;
 public:
   cSkinClassicDisplayReplay(bool ModeOnly);
   virtual ~cSkinClassicDisplayReplay();
@@ -397,7 +407,6 @@ cSkinClassicDisplayReplay::cSkinClassicDisplayReplay(bool ModeOnly)
   const cFont *font = cFont::GetFont(fontOsd);
   int lineHeight = font->Height();
   lastCurrentWidth = 0;
-  message = false;
   x0 = 0;
   x1 = Setup.OSDWidth;
   y0 = 0;
@@ -467,12 +476,9 @@ void cSkinClassicDisplayReplay::SetMessage(eMessageType Type, const char *Text)
   if (Text) {
      osd->SaveRegion(x0, y2, x1 - 1, y3 - 1);
      osd->DrawText(x0, y2, Text, Theme.Color(clrMessageStatusFg + 2 * Type), Theme.Color(clrMessageStatusBg + 2 * Type), font, x1 - x0, y3 - y2, taCenter);
-     message = true;
      }
-  else {
+  else
      osd->RestoreRegion();
-     message = false;
-     }
 }
 
 void cSkinClassicDisplayReplay::Flush(void)
@@ -511,7 +517,7 @@ void cSkinClassicDisplayVolume::SetVolume(int Current, int Total, bool Mute)
   const cFont *font = cFont::GetFont(fontOsd);
   if (Mute) {
      osd->DrawRectangle(0, 0, osd->Width() - 1, osd->Height() - 1, clrTransparent);
-     osd->DrawText(0, 0, tr("Mute"), Theme.Color(clrVolumePrompt), Theme.Color(clrBackground), font);
+     osd->DrawText(0, 0, tr("Key$Mute"), Theme.Color(clrVolumePrompt), Theme.Color(clrBackground), font);
      }
   else {
      const char *Prompt = tr("Volume ");
