@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: skinsttng.c 1.15 2005/08/15 11:14:59 kls Exp $
+ * $Id: skinsttng.c 1.19 2006/02/17 15:57:37 kls Exp $
  */
 
 // Star Trek: The Next Generation is a registered trademark of Paramount Pictures
@@ -347,6 +347,8 @@ public:
   virtual void SetEvent(const cEvent *Event);
   virtual void SetRecording(const cRecording *Recording);
   virtual void SetText(const char *Text, bool FixedFont);
+  virtual int GetTextAreaWidth(void) const;
+  virtual const cFont *GetTextAreaFont(bool FixedFont) const;
   virtual void Flush(void);
   };
 
@@ -618,12 +620,22 @@ void cSkinSTTNGDisplayMenu::SetRecording(const cRecording *Recording)
 
 void cSkinSTTNGDisplayMenu::SetText(const char *Text, bool FixedFont)
 {
-  const cFont *font = cFont::GetFont(FixedFont ? fontFix : fontOsd);
-  font = cFont::GetFont(fontSml);//XXX -> make a way to let the text define which font to use
-  textScroller.Set(osd, x3, y3, x4 - x3, y4 - y3, Text, font, Theme.Color(clrMenuText), Theme.Color(clrBackground));
+  textScroller.Set(osd, x3, y3, GetTextAreaWidth(), y4 - y3, Text, GetTextAreaFont(FixedFont), Theme.Color(clrMenuText), Theme.Color(clrBackground));
   SetScrollbar();
 }
 
+int cSkinSTTNGDisplayMenu::GetTextAreaWidth(void) const
+{
+  return x4 - x3;
+}
+
+const cFont *cSkinSTTNGDisplayMenu::GetTextAreaFont(bool FixedFont) const
+{
+  const cFont *font = cFont::GetFont(FixedFont ? fontFix : fontOsd);
+  //XXX -> make a way to let the text define which font to use
+  return font;
+}
+
 void cSkinSTTNGDisplayMenu::Flush(void)
 {
   if (!message) {
@@ -643,7 +655,6 @@ private:
   int y0, y1, y2, y3, y4, y5, y6, y7;
   tColor frameColor;
   int lastCurrentWidth;
-  bool message;
 public:
   cSkinSTTNGDisplayReplay(bool ModeOnly);
   virtual ~cSkinSTTNGDisplayReplay();
@@ -666,7 +677,6 @@ cSkinSTTNGDisplayReplay::cSkinSTTNGDisplayReplay(bool ModeOnly)
   int lineHeight = font->Height();
   frameColor = Theme.Color(clrReplayFrame);
   lastCurrentWidth = 0;
-  message = false;
   cBitmap bm(play_xpm);
   x0 = 0;
   x1 = max(SymbolWidth, bm.Width());
@@ -723,7 +733,7 @@ void cSkinSTTNGDisplayReplay::SetTitle(const char *Title)
   osd->DrawText(x3 + 5, y0, Title, Theme.Color(clrReplayTitle), frameColor, cFont::GetFont(fontSml), x4 - x3 - 5);
 }
 
-static char **ReplaySymbols[2][2][5] = {
+static const char *const *ReplaySymbols[2][2][5] = {
   { { pause_xpm, srew_xpm, srew1_xpm, srew2_xpm, srew3_xpm },
     { pause_xpm, sfwd_xpm, sfwd1_xpm, sfwd2_xpm, sfwd3_xpm }, },
   { { play_xpm,  frew_xpm, frew1_xpm, frew2_xpm, frew3_xpm },
@@ -772,12 +782,9 @@ void cSkinSTTNGDisplayReplay::SetMessage(eMessageType Type, const char *Text)
      osd->SaveRegion(x2, y6, x4 - 1, y7 - 1);
      osd->DrawRectangle(x2, y6, x3 - 1, y7 - 1, Theme.Color(clrBackground));
      osd->DrawText(x3, y6, Text, Theme.Color(clrMessageStatusFg + 2 * Type), Theme.Color(clrMessageStatusBg + 2 * Type), font, x4 - x3, 0, taCenter);
-     message = true;
      }
-  else {
+  else
      osd->RestoreRegion();
-     message = false;
-     }
 }
 
 void cSkinSTTNGDisplayReplay::Flush(void)
