@@ -10,7 +10,7 @@
  * and interact with the Video Disk Recorder - or write a full featured
  * graphical interface that sits on top of an SVDRP connection.
  *
- * $Id: svdrp.c 1.95 2006/04/17 09:02:23 kls Exp $
+ * $Id: svdrp.c 1.100 2006/08/12 09:09:55 kls Exp $
  */
 
 #include "svdrp.h"
@@ -120,7 +120,7 @@ int cSocket::Accept(void)
            close(newsock);
            newsock = -1;
            }
-        isyslog("connect from %s, port %hd - %s", inet_ntoa(clientname.sin_addr), ntohs(clientname.sin_port), accepted ? "accepted" : "DENIED");
+        isyslog("connect from %s, port %hu - %s", inet_ntoa(clientname.sin_addr), ntohs(clientname.sin_port), accepted ? "accepted" : "DENIED");
         }
      else if (errno != EINTR && errno != EAGAIN)
         LOG_ERROR;
@@ -378,17 +378,19 @@ cSVDRP::cSVDRP(int Port)
 
 cSVDRP::~cSVDRP()
 {
-  Close();
+  Close(true);
   free(cmdLine);
 }
 
-void cSVDRP::Close(bool Timeout)
+void cSVDRP::Close(bool SendReply, bool Timeout)
 {
   if (file.IsOpen()) {
-     //TODO how can we get the *full* hostname?
-     char buffer[BUFSIZ];
-     gethostname(buffer, sizeof(buffer));
-     Reply(221, "%s closing connection%s", buffer, Timeout ? " (timeout)" : "");
+     if (SendReply) {
+        //TODO how can we get the *full* hostname?
+        char buffer[BUFSIZ];
+        gethostname(buffer, sizeof(buffer));
+        Reply(221, "%s closing connection%s", buffer, Timeout ? " (timeout)" : "");
+        }
      isyslog("closing SVDRP connection"); //TODO store IP#???
      file.Close();
      DELETENULL(PUTEhandler);
@@ -401,7 +403,7 @@ bool cSVDRP::Send(const char *s, int length)
      length = strlen(s);
   if (safe_write(file, s, length) < 0) {
      LOG_ERROR;
-     file.Close();
+     Close();
      return false;
      }
   return true;
@@ -423,10 +425,8 @@ void cSVDRP::Reply(int Code, const char *fmt, ...)
                  cont = '-';
               char number[16];
               sprintf(number, "%03d%c", abs(Code), cont);
-              if (!(Send(number) && Send(s, n ? n - s : -1) && Send("\r\n"))) {
-                 Close();
+              if (!(Send(number) && Send(s, n ? n - s : -1) && Send("\r\n")))
                  break;
-                 }
               s = n ? n + 1 : NULL;
               }
         free(buffer);
@@ -550,10 +550,25 @@ void cSVDRP::CmdDELC(const char *Option)
                      return;
                      }
                   }
+              int CurrentChannelNr = cDevice::CurrentChannel();
+              cChannel *CurrentChannel = Channels.GetByNumber(CurrentChannelNr);
+              if (CurrentChannel && channel == CurrentChannel) {
+                 int n = Channels.GetNextNormal(CurrentChannel->Index());
+                 if (n < 0)
+                    n = Channels.GetPrevNormal(CurrentChannel->Index());
+                 CurrentChannel = Channels.Get(n);
+                 CurrentChannelNr = 0; // triggers channel switch below
+                 }
               Channels.Del(channel);
               Channels.ReNumber();
               Channels.SetModified(true);
               isyslog("channel %s deleted", Option);
+              if (CurrentChannel && CurrentChannel->Number() != CurrentChannelNr) {
+                 if (!cDevice::PrimaryDevice()->Replaying() || cDevice::PrimaryDevice()->Transferring())
+                    Channels.SwitchTo(CurrentChannel->Number());
+                 else
+                    cDevice::SetCurrentChannel(CurrentChannel);
+                 }
               Reply(250, "Channel \"%s\" deleted", Option);
               }
            else
@@ -764,7 +779,7 @@ void cSVDRP::CmdGRAB(const char *Option)
            int fd = open(FileName, O_WRONLY | O_CREAT | O_NOFOLLOW | O_TRUNC, DEFFILEMODE);
            if (fd >= 0) {
               if (safe_write(fd, Image, ImageSize) == ImageSize) {
-                 isyslog("grabbed image to %s", FileName);
+                 dsyslog("grabbed image to %s", FileName);
                  Reply(250, "Grabbed image %s", Option);
                  }
               else {
@@ -1158,8 +1173,12 @@ void cSVDRP::CmdMOVC(const char *Option)
                        Channels.Move(FromChannel, ToChannel);
                        Channels.ReNumber();
                        Channels.SetModified(true);
-                       if (CurrentChannel && CurrentChannel->Number() != CurrentChannelNr)
-                          Channels.SwitchTo(CurrentChannel->Number());
+                       if (CurrentChannel && CurrentChannel->Number() != CurrentChannelNr) {
+                          if (!cDevice::PrimaryDevice()->Replaying() || cDevice::PrimaryDevice()->Transferring())
+                             Channels.SwitchTo(CurrentChannel->Number());
+                          else
+                             cDevice::SetCurrentChannel(CurrentChannel);
+                          }
                        isyslog("channel %d moved to %d", FromNumber, ToNumber);
                        Reply(250,"Channel \"%d\" moved to \"%d\"", From, To);
                        }
@@ -1511,7 +1530,7 @@ void cSVDRP::Execute(char *Cmd)
   else if (CMD("STAT"))  CmdSTAT(s);
   else if (CMD("UPDT"))  CmdUPDT(s);
   else if (CMD("VOLU"))  CmdVOLU(s);
-  else if (CMD("QUIT"))  Close();
+  else if (CMD("QUIT"))  Close(true);
   else                   Reply(500, "Command unrecognized: \"%s\"", Cmd);
 }
 
@@ -1551,7 +1570,7 @@ bool cSVDRP::Process(void)
                  }
               else if (c == 0x04 && numChars == 0) {
                  // end of file (only at beginning of line)
-                 Close();
+                 Close(true);
                  }
               else if (c == 0x08 || c == 0x7F) {
                  // backspace or delete (last character)
@@ -1571,16 +1590,14 @@ bool cSVDRP::Process(void)
                  }
               lastActivity = time(NULL);
               }
-           else if (r < 0) {
+           else if (r <= 0) {
               isyslog("lost connection to SVDRP client");
               Close();
               }
-           else
-              break;
            }
      if (Setup.SVDRPTimeout && time(NULL) - lastActivity > Setup.SVDRPTimeout) {
         isyslog("timeout on SVDRP connection");
-        Close(true);
+        Close(true, true);
         }
      return true;
      }
